#version 150
precision highp float; precision highp int;
//Ethan Alexander Shulman 2022 "Beep Bop" November 19, 2022

in vec3 in_Position;
out vec4 uv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};
layout(std140) uniform MidiBlock {
	vec4 midi[64];//x = note id, y = velocity, z = channel/track, w = time in seconds since note pressed
	int midiCount;
};
/*Midi Instrument 0 = Warm Stack
Midi Instrument 1 = TR909_Kick.ds
Midi Instrument 2 = Kick_1.ds
Midi Instrument 3 = Broken Toy*/
out vec4 music;//x = warm stack, y = kick, z = broken toy

#define TEMPO_TIME(bpm,note) (60.*4.*note/bpm)


//#define SCREENSHOT 1
#define AA_SAMPLES 8


//r2 hash http://extremelearning.com.au/unreasonable-effectiveness-of-quasirandom-sequences/
vec2 quasiRandom(float n) {
	#define R2V 1.32471795724474602596
	return fract(.5+n*(1./vec2(R2V,R2V*R2V)));
}

float light(vec2 p, vec2 s) {
	p = max(abs(uv.xy-p+vec2((fract(abs(uv.y)*552.78612)-.5)*.12,0))-s,0.);
	return 1./(1.+1000.*dot(p,p));
}


uniform sampler2D Buffer0;

void vert() {
	uv = vec4((in_Position.xy*vec2(screenX/screenY,1)+(quasiRandom(float(frame%AA_SAMPLES))-.5)*2./screenY),
		(in_Position.xy*.5+.5)*vec2(screenX,screenY));
	#ifdef SCREENSHOT
	time = 0.;
	music = vec4(1);
	#else
	time = float(frame)/float(framerate*AA_SAMPLES);
	for (int i = 0; i < midiCount; i++) {
		float v = midi[i].z;
		if (v == 0.) music.x = 1.;
		else if (v == 1. && midi[i].w < .25) music.y = 1.;
		else if (v == 3. && fract(midi[i].w/TEMPO_TIME(120.,1./16.)) < .4) music.z = 1.;
	}
	#endif

	gl_Position = vec4(in_Position,1);
}
void frag() {
	vec3 c = vec3(0);
	c += vec3(.3,.5,.9)*light(vec2(.5,.22),vec2(.16))*music.x;
	c += vec3(.2,1,.5)*light(vec2(.5,-.22),vec2(.16))*music.x;
	
	c += vec3(1,1,0)*light(vec2(0,.22),vec2(.16))*music.y;
	c += vec3(1,.5,0)*light(vec2(0,-.22),vec2(.16))*music.y;
		
	c += vec3(1,0,0)*light(vec2(-.5,0),vec2(.16,.4))*music.z;
	
	vec4 sum = texelFetch(Buffer0,ivec2(uv.zw),0);
	if (frame%AA_SAMPLES == 0) sum = vec4(0);
	sum += vec4(c, 1.);
	gl_FragColor = sum;
}
